@@ -1104,10 +1104,12 @@ static const struct
   "artist", GST_TAG_ARTIST}, {
   "comment", GST_TAG_COMMENT}, {
   "composer", GST_TAG_COMPOSER}, {
-  "copyright", GST_TAG_COPYRIGHT}, {
-    /* Need to convert ISO 8601 to GstDateTime: */
-  "creation_time", GST_TAG_DATE_TIME}, {
-    /* Need to convert ISO 8601 to GDateTime: */
+  "copyright", GST_TAG_COPYRIGHT},
+      /* Need to convert ISO 8601 to GstDateTime: */
+  {
+  "creation_time", GST_TAG_DATE_TIME},
+      /* Need to convert ISO 8601 to GDateTime: */
+  {
   "date", GST_TAG_DATE_TIME}, {
   "disc", GST_TAG_ALBUM_VOLUME_NUMBER}, {
   "encoder", GST_TAG_ENCODER}, {
@@ -1738,6 +1740,7 @@ gst_ffmpegdemux_sink_event (GstPad * sinkpad, GstObject * parent,
       goto done;
     case GST_EVENT_STREAM_START:
     case GST_EVENT_CAPS:
+    case GST_EVENT_SEGMENT:
       GST_LOG_OBJECT (demux, "dropping %s event", GST_EVENT_TYPE_NAME (event));
       gst_event_unref (event);
       goto done;
